# -*- coding: utf-8 -*-
import datetime

from django.db import models
from django.utils.translation import ugettext_lazy as _
from django.contrib.auth.models import User
from django.conf import settings
from django.db.models import Q

from sorl.thumbnail.fields import ThumbnailField
from jumbo.middleware.thread_local import get_request
from jumbo.db.models import DateModel, UserModel, OrderedModel, StatusModel, HTMLField
from jumbo.apps.i18n.models import TranslationModel, I18NModel

##############################
# TIPO NEWS                  #
##############################
class CategoryManager(models.Manager):
    def default(self):
        return self.get_query_set().get(default=True)

class Category(DateModel, UserModel, StatusModel, OrderedModel):
    objects = CategoryManager()

    ## campi modello
    name = models.CharField(
        unique=True,
        max_length=25,
        verbose_name=_('News type'),
    )
    default = models.BooleanField(
        default=False,
        verbose_name=_('Default category'),
        help_text=_('Is the default one?'),
    )

    def __unicode__(self):
        return u"%s" % self.name

    class Meta:
        verbose_name = _('News type')
        verbose_name_plural = _('News types')

    def save(self, force_insert=False, force_update=False):
        super(Category, self).save(force_insert=force_insert, force_update=force_update)
        if self.default:
            for sc in Category.objects.exclude(pk=self.id):
                sc.default = False
                sc.save(force_insert=force_insert, force_update=force_update)
        try:
            sc = Category.objects.get(default=True)
        except models.ObjectDoesNotExist:
            self.default = True
            super(Category, self).save(force_insert=force_insert, force_update=force_update)


##########################
# NEWS                   #
##########################
class ActiveNewsManager(models.Manager):
    def get_query_set(self):
        now = datetime.datetime.now()
        q1 = Q(pub_date_begin__lte=now) | Q(pub_date_begin__isnull=True)
        q2 = Q(pub_date_end__gte=now) | Q(pub_date_end__isnull=True)
        return super(ActiveNewsManager, self).get_query_set().filter(status=1).filter(q1 & q2)


class News(DateModel, UserModel, OrderedModel, StatusModel, I18NModel):
    ## ForeignKey
    categories = models.ManyToManyField(
        Category,
        null=True,
        blank=True,
        related_name = "news",
        verbose_name = _('Categories')
    )
    users = models.ManyToManyField(
        User,
        blank=True,
        null=True,
        related_name='news',
        verbose_name=_('Visible only to'),
        help_text=_('News visible to these users, if empty is visible to all users.'),
    )
    ## Campi
    name = models.CharField(
        unique=True,
        max_length=100,
        verbose_name=_('News'),
    )
    event_date = models.DateField(
        blank=True,
        null=True,
        verbose_name=_('News date'),
        help_text=_('Date which news refers to.'),
    )
    pub_date_begin = models.DateTimeField(
        blank=True,
        null=True,
        verbose_name=_('Publication date begin'),
        help_text=_('When news publication date begins.'),
    )
    pub_date_end = models.DateTimeField(
        blank=True,
        null=True,
        verbose_name=_('Publication date end'),
        help_text=_('When news publication date ends.'),
    )
    title = models.CharField(
        max_length=255,
        blank=True,
        null=True,
        verbose_name=_('Default title'),
    )
    small_image = ThumbnailField(
        upload_to='news/thumbnail',
        size=(200, 150),
        blank = True, null = True,
        verbose_name = _("Small Image")
    )
    big_image = ThumbnailField(
        upload_to='news/bigimage',
        size=(700, 1500),
        blank = True, null = True,
        verbose_name = _("Big Image")
    )
    short_description = models.CharField(
        max_length=100,
        verbose_name=_('Short Description'),
        blank = True, null = True
    )
    text = HTMLField(
        blank=True,
        null=True,
        verbose_name=_('Long Description'),
    )
    note = models.TextField(
        blank=True,
        verbose_name=_('Note'),
    )
    objects = models.Manager() # The default manager.
    active = ActiveNewsManager()


    def save(self, *args, **kwargs):
        if not self.pk and not self.title:
            self.title = self.name
        super(News, self).save(*args, **kwargs)


    def get_users(self):
        users = self.users.all().order_by('username')
        if users:
            return u', '.join([user.username for user in users])
        else:
            return _(u'All')
    get_users.short_description = _('show to')


    def get_categories(self):
        categories = self.categories.all().order_by('name')
        if categories:
            return u', '.join([category.name for category in categories])
        else:
            return _(u'All')
    get_categories.short_description = _('Categories')


    def get_short_description(self):
        return self.short_description or "%s ....." % self.text[:100]

    class Meta:
        verbose_name = _("News")
        verbose_name_plural = _("News")

    def __unicode__(self):
        if self.title:
            return self.title
        return self.name

##########################
# NEWS TRANSLATION       #
##########################
class NewsTranslation(DateModel, UserModel, StatusModel, TranslationModel):
    ## ForeignKey
    news = models.ForeignKey(
        News,
        related_name='translations',
        verbose_name=_('News type'),
    )
    title = models.CharField(
        max_length=256,
        verbose_name=_('Title'),
    )
    text = HTMLField(
        verbose_name=_('Body'),
    )

    class Meta:
        verbose_name = _("News translation")
        verbose_name_plural = _("News translations")
        unique_together = (('news','language',),)

    def __unicode__(self):
        return u'%s (%s)' % (self.news.title, self.language)

