# -*- coding: utf-8 -*-

import datetime
from django import template
from django.conf import settings
from django.db.models import Q

from django.utils.importlib import import_module
models = import_module('news.models')

register = template.Library()

@register.inclusion_tag('news/tags/latest_news_list.html', takes_context=True)
def list_latest_news(context, num_news=None, category=None, sort='DESC'):
    kwargs = {}
    if category:
        kwargs.update({'categories__name__in': [category]})
    news = models.News.active.filter(**kwargs)

    q1 = Q(pub_date_begin__lte=datetime.datetime.now(), pub_date_end__gte=datetime.datetime.now())
    q2 = Q(event_date__lte=datetime.datetime.now())

    order_by = 'event_date'
    if sort=='DESC':
        order_by = '-event_date'

    context['latest_news'] = news.filter(
        q1 | q2
    ).order_by(order_by)
    if num_news:
        context['latest_news'] = context['latest_news'][:num_news]

    context['category'] = category

    return context


@register.inclusion_tag('news/tags/year_archive.html', takes_context=True)
def list_year_archive(context, category=None, year=None, sort='DESC'):
    kwargs = {}
    year = datetime.date.today().year
    #if year:
    #    kwargs.update({'event_date__year': year})
    if category:
        kwargs.update({'categories__name__in': [category]})

    # filter by pubblication date
    context['month_list'] = models.News.active.filter(**kwargs).dates('event_date', 'month', order=sort)[:12]
    context['category'] = category

    return context



class LatestNewsNode(template.Node):
    def __init__(self, num_news, categoria):
        self.num_news = num_news
        self.categoria = categoria

    def render(self, context):
        q1 = Q(pub_date_begin__lte=datetime.datetime.now(), pub_date_end__gte=datetime.datetime.now())
        q2 = Q(event_date__lte=datetime.datetime.now())
        kwargs = {}
        if self.categoria:
            kwargs.update({'categories__name__in': [self.categoria]})
        context['latest_news'] = models.News.active.filter(**kwargs).filter(
            q1 | q2
        ).order_by('-event_date')[:self.num_news]
        return ''

@register.tag
def get_latest_news(parser, token):
    """
        Possibili usi:
        {% get_latest_news %} --> Torna le ultime 15 news di tutte le categorie
        {% get_latest_news 5 %} --> Torna le ultime 5 news di tutte le categorie
        {% get_latest_news 5 'EVENTI' %} --> Torna le ultime 5 news della categoria EVENTI
    """
    try:
        contents = token.split_contents()
        num_news = 15
        categoria = None
        if len(contents) >= 2:
            num_news = int(contents[1])
            if len(contents) == 3:
                categoria = contents[2]
    except Exception, e:
        raise template.TemplateSyntaxError, "%r tag requires max only two arguments: num_news as integer and category as string" % token.contents.split()[0]
    return LatestNewsNode(num_news, categoria)


class MonthListNode(template.Node):
    def __init__(self, year):
        self.year = year

    def render(self, context):
        queryset = models.News.active.filter(event_date__year=self.year)
        context['month_news_list'] = queryset.dates('event_date', 'month')
        return ''
@register.tag
def get_month_news_list(parser, token):
    try:
        contents = token.split_contents()
        year = datetime.date.today().year
        if len(contents) == 2:
            year = int(contents[1])
    except Exception, e:
        raise template.TemplateSyntaxError, "%r tag requires only one argument year as integer" % token.contents.split()[0]
    return MonthListNode(year)


class LastNewsNode(template.Node):
    def __init__(self, category):
        self.category = category

    def render(self, context):
        qs = models.News.active.all()
        if self.category:
            qs = qs.filter(categories__name__contains=self.category)

        news = None
        q1 = Q(pub_date_begin__lte=datetime.datetime.now(), pub_date_end__gte=datetime.datetime.now())
        q2 = Q(event_date__gte=datetime.datetime.now())
        q3 = Q(event_date__lte=datetime.datetime.now())

        try:
            # Cerco la prima news di oggi o futura
            news = qs.filter(
                q1 | q2
            ).order_by('event_date')[0:1].get()
        except models.News.DoesNotExist:
            # Se non la trovo prendo la prima news del passato
            news = qs.filter(
                q1 | q3
            ).order_by('-event_date')
            if news:
                news = news[0]

        context['last_news'] = news
        return ''
@register.tag
def get_last_news(parser, token):
    try:
        contents = token.split_contents()
        category = None
        if len(contents) == 2:
            category = contents[1][1:-1]
    except:
        raise template.TemplateSyntaxError, "%r tag requires only one argument named category" % token.contents.split()[0]
    return LastNewsNode(category)